/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

/**
 * @author Alexander Y. Kleymenov
 */

package org.apache.harmony.security.tests.support.provider.cert;

import java.io.UnsupportedEncodingException;

import org.apache.harmony.luni.util.Base64;

/**
 * Class contains the base 64 encodings of X.509 certificates,
 * CRLs, and Certification Paths.
 */
public class CertFactoryTestData {

    // All the testing data was generated by using of classes 
    // from org.apache.harmony.security.asn1 package and encoded
    // by org.apache.harmony.misc.Base64 class.

    private static String certPathPKCS7Base64 =
            "MIIJXQYJKoZIhvcNAQcCoIIJTjCCCUoCAQExADALBgkqhkiG9w" +
                    "0BBwGgggkyMIIElTCCBFKgAwIBAgICAiswDAYHKoZIzjgEAwEB" +
                    "ADAdMRswGQYDVQQKExJDZXJ0aWZpY2F0ZSBJc3N1ZXIwHhcNNz" +
                    "AwMTEyMTM0NjQwWhcNNzAwMTI0MDMzMzIwWjAfMR0wGwYDVQQK" +
                    "ExRTdWJqZWN0IE9yZ2FuaXphdGlvbjCCAbcwggEsBgcqhkjOOA" +
                    "QBMIIBHwKBgQD9f1OBHXUSKVLfSpwu7OTn9hG3UjzvRADDHj+A" +
                    "tlEmaUVdQCJR+1k9jVj6v8X1ujD2y5tVbNeBO4AdNG/yZmC3a5" +
                    "lQpaSfn+gEexAiwk+7qdf+t8Yb+DtX58aophUPBPuD9tPFHsMC" +
                    "NVQTWhaRMvZ1864rYdcq7/IiAxmd0UgBxwIVAJdgUI8VIwvMsp" +
                    "K5gqLrhAvwWBz1AoGBAPfhoIXWmz3ey7yrXDa4V7l5lK+7+jrq" +
                    "gvlXTAs9B4JnUVlXjrrUWU/mcQcQgYC0SRZxI+hMKBYTt88JMo" +
                    "zIpuE8FnqLVHyNKOCjrh4rs6Z1kW6jfwv6ITVi8ftiegEkO8yk" +
                    "8b6oUZCJqIPf4VrlnwaSi2ZegHtVJWQBTDv+z0kqA4GEAAKBgB" +
                    "DIG+uTY/ABkBHadFyFnYoqaxprR5U6bZVMPmCBdnxh1dGmiah6" +
                    "x2f+lmvEyPha1oNkdesDIBNId1Akkt/sQnCA6C9q038VyfmMPb" +
                    "6khvtlZ9mCEiV4D72zieYhbiHzBmJsPKC8JQ/xFJwSU0e//2bl" +
                    "uF+UGjbA4/WMLi3+Vf9hgQIAqoICAFWjggIUMIICEDAPBgNVHQ" +
                    "8BAf8EBQMDAaqAMBIGA1UdEwEB/wQIMAYBAf8CAQUwFAYDVR0g" +
                    "AQH/BAowCDAGBgRVHSAAMGcGA1UdEQEB/wRdMFuBDHJmY0A4Mj" +
                    "IuTmFtZYIHZE5TTmFtZaQXMRUwEwYDVQQKEwxPcmdhbml6YXRp" +
                    "b26GGmh0dHA6Ly91bmlmb3JtLlJlc291cmNlLklkhwT///8AiA" +
                    "cqA6Jcg7IDMAwGA1UdHgEB/wQCMAAwDAYDVR0kAQH/BAIwADCB" +
                    "mQYDVR0lAQH/BIGOMIGLBgRVHSUABggrBgEFBQcDAQYIKwYBBQ" +
                    "UHAwEGCCsGAQUFBwMCBggrBgEFBQcDAwYIKwYBBQUHAwQGCCsG" +
                    "AQUFBwMFBggrBgEFBQcDBgYIKwYBBQUHAwcGCCsGAQUFBwMIBg" +
                    "grBgEFBQcDCQYIKwYBBQUIAgIGCisGAQQBgjcKAwMGCWCGSAGG" +
                    "+EIEATANBgNVHTYBAf8EAwIBATAOBgQqTYYJAQH/BAMBAQEwZA" +
                    "YDVR0SBF0wW4EMcmZjQDgyMi5OYW1lggdkTlNOYW1lpBcxFTAT" +
                    "BgNVBAoTDE9yZ2FuaXphdGlvboYaaHR0cDovL3VuaWZvcm0uUm" +
                    "Vzb3VyY2UuSWSHBP///wCIByoDolyDsgMwCQYDVR0fBAIwADAK" +
                    "BgNVHSMEAwEBATAKBgNVHQ4EAwEBATAKBgNVHSEEAwEBATAMBg" +
                    "cqhkjOOAQDAQEAAy8AMCwCFDTTeOfOqTwbBYYP3bPIKY6ctqxb" +
                    "AhQq7Qv07HsN3LJQCUPoQdMTx6GjyDCCBJUwggRSoAMCAQICAg" +
                    "IrMAwGByqGSM44BAMBAQAwHTEbMBkGA1UEChMSQ2VydGlmaWNh" +
                    "dGUgSXNzdWVyMB4XDTcwMDExMjEzNDY0MFoXDTcwMDEyNDAzMz" +
                    "MyMFowHzEdMBsGA1UEChMUU3ViamVjdCBPcmdhbml6YXRpb24w" +
                    "ggG3MIIBLAYHKoZIzjgEATCCAR8CgYEA/X9TgR11EilS30qcLu" +
                    "zk5/YRt1I870QAwx4/gLZRJmlFXUAiUftZPY1Y+r/F9bow9sub" +
                    "VWzXgTuAHTRv8mZgt2uZUKWkn5/oBHsQIsJPu6nX/rfGG/g7V+" +
                    "fGqKYVDwT7g/bTxR7DAjVUE1oWkTL2dfOuK2HXKu/yIgMZndFI" +
                    "AccCFQCXYFCPFSMLzLKSuYKi64QL8Fgc9QKBgQD34aCF1ps93s" +
                    "u8q1w2uFe5eZSvu/o66oL5V0wLPQeCZ1FZV4661FlP5nEHEIGA" +
                    "tEkWcSPoTCgWE7fPCTKMyKbhPBZ6i1R8jSjgo64eK7OmdZFuo3" +
                    "8L+iE1YvH7YnoBJDvMpPG+qFGQiaiD3+Fa5Z8GkotmXoB7VSVk" +
                    "AUw7/s9JKgOBhAACgYAQyBvrk2PwAZAR2nRchZ2KKmsaa0eVOm" +
                    "2VTD5ggXZ8YdXRpomoesdn/pZrxMj4WtaDZHXrAyATSHdQJJLf" +
                    "7EJwgOgvatN/Fcn5jD2+pIb7ZWfZghIleA+9s4nmIW4h8wZibD" +
                    "ygvCUP8RScElNHv/9m5bhflBo2wOP1jC4t/lX/YYECAKqCAgBV" +
                    "o4ICFDCCAhAwDwYDVR0PAQH/BAUDAwGqgDASBgNVHRMBAf8ECD" +
                    "AGAQH/AgEFMBQGA1UdIAEB/wQKMAgwBgYEVR0gADBnBgNVHREB" +
                    "Af8EXTBbgQxyZmNAODIyLk5hbWWCB2ROU05hbWWkFzEVMBMGA1" +
                    "UEChMMT3JnYW5pemF0aW9uhhpodHRwOi8vdW5pZm9ybS5SZXNv" +
                    "dXJjZS5JZIcE////AIgHKgOiXIOyAzAMBgNVHR4BAf8EAjAAMA" +
                    "wGA1UdJAEB/wQCMAAwgZkGA1UdJQEB/wSBjjCBiwYEVR0lAAYI" +
                    "KwYBBQUHAwEGCCsGAQUFBwMBBggrBgEFBQcDAgYIKwYBBQUHAw" +
                    "MGCCsGAQUFBwMEBggrBgEFBQcDBQYIKwYBBQUHAwYGCCsGAQUF" +
                    "BwMHBggrBgEFBQcDCAYIKwYBBQUHAwkGCCsGAQUFCAICBgorBg" +
                    "EEAYI3CgMDBglghkgBhvhCBAEwDQYDVR02AQH/BAMCAQEwDgYE" +
                    "Kk2GCQEB/wQDAQEBMGQGA1UdEgRdMFuBDHJmY0A4MjIuTmFtZY" +
                    "IHZE5TTmFtZaQXMRUwEwYDVQQKEwxPcmdhbml6YXRpb26GGmh0" +
                    "dHA6Ly91bmlmb3JtLlJlc291cmNlLklkhwT///8AiAcqA6Jcg7" +
                    "IDMAkGA1UdHwQCMAAwCgYDVR0jBAMBAQEwCgYDVR0OBAMBAQEw" +
                    "CgYDVR0hBAMBAQEwDAYHKoZIzjgEAwEBAAMvADAsAhQ003jnzq" +
                    "k8GwWGD92zyCmOnLasWwIUKu0L9Ox7DdyyUAlD6EHTE8eho8gx" +
                    "AA==";

    private static String certPathPkiPathBase64 =
            "MIIJNjCCBJcwggRToAMCAQICAgIrMAwGByqGSM44BAMBAQAwHT" +
                    "EbMBkGA1UEChMSQ2VydGlmaWNhdGUgSXNzdWVyMB4XDTcwMDEx" +
                    "MjEzNDY0MFoXDTcwMDEyNDAzMzMyMFowHzEdMBsGA1UEChMUU3" +
                    "ViamVjdCBPcmdhbml6YXRpb24wggG4MIIBLAYHKoZIzjgEATCC" +
                    "AR8CgYEA/X9TgR11EilS30qcLuzk5/YRt1I870QAwx4/gLZRJm" +
                    "lFXUAiUftZPY1Y+r/F9bow9subVWzXgTuAHTRv8mZgt2uZUKWk" +
                    "n5/oBHsQIsJPu6nX/rfGG/g7V+fGqKYVDwT7g/bTxR7DAjVUE1" +
                    "oWkTL2dfOuK2HXKu/yIgMZndFIAccCFQCXYFCPFSMLzLKSuYKi" +
                    "64QL8Fgc9QKBgQD34aCF1ps93su8q1w2uFe5eZSvu/o66oL5V0" +
                    "wLPQeCZ1FZV4661FlP5nEHEIGAtEkWcSPoTCgWE7fPCTKMyKbh" +
                    "PBZ6i1R8jSjgo64eK7OmdZFuo38L+iE1YvH7YnoBJDvMpPG+qF" +
                    "GQiaiD3+Fa5Z8GkotmXoB7VSVkAUw7/s9JKgOBhQACgYEAzhsi" +
                    "Nfq3U9RAWhut8YDmRbHJTXMHTytGsK0iRZqg47JllyWbHwSmS8" +
                    "T2k4Dk4FYxQzxgOxW/PbZ5c06P9BYPlazQTcPIwAWSlKTaoTfn" +
                    "WzHnp75hbbPc8HmrSIEHVPr5hHS0DQl5Fu45b5U2KG5cx2Bb3l" +
                    "k17n0y/RqFXFYTv+OBAgCqggIAVaOCAhQwggIQMA8GA1UdDwEB" +
                    "/wQFAwMBqoAwEgYDVR0TAQH/BAgwBgEB/wIBBTAUBgNVHSABAf" +
                    "8ECjAIMAYGBFUdIAAwZwYDVR0RAQH/BF0wW4EMcmZjQDgyMi5O" +
                    "YW1lggdkTlNOYW1lpBcxFTATBgNVBAoTDE9yZ2FuaXphdGlvbo" +
                    "YaaHR0cDovL3VuaWZvcm0uUmVzb3VyY2UuSWSHBP///wCIByoD" +
                    "olyDsgMwDAYDVR0eAQH/BAIwADAMBgNVHSQBAf8EAjAAMIGZBg" +
                    "NVHSUBAf8EgY4wgYsGBFUdJQAGCCsGAQUFBwMBBggrBgEFBQcD" +
                    "AQYIKwYBBQUHAwIGCCsGAQUFBwMDBggrBgEFBQcDBAYIKwYBBQ" +
                    "UHAwUGCCsGAQUFBwMGBggrBgEFBQcDBwYIKwYBBQUHAwgGCCsG" +
                    "AQUFBwMJBggrBgEFBQgCAgYKKwYBBAGCNwoDAwYJYIZIAYb4Qg" +
                    "QBMA0GA1UdNgEB/wQDAgEBMA4GBCpNhgkBAf8EAwEBATBkBgNV" +
                    "HRIEXTBbgQxyZmNAODIyLk5hbWWCB2ROU05hbWWkFzEVMBMGA1" +
                    "UEChMMT3JnYW5pemF0aW9uhhpodHRwOi8vdW5pZm9ybS5SZXNv" +
                    "dXJjZS5JZIcE////AIgHKgOiXIOyAzAJBgNVHR8EAjAAMAoGA1" +
                    "UdIwQDAQEBMAoGA1UdDgQDAQEBMAoGA1UdIQQDAQEBMAwGByqG" +
                    "SM44BAMBAQADMAAwLQIVAIu6VjdQkOyX3jwUCvZk7i8ascXxAh" +
                    "Qb4oUMkiRQ1/ThtGvgtiuQUzHBZDCCBJcwggRToAMCAQICAgIr" +
                    "MAwGByqGSM44BAMBAQAwHTEbMBkGA1UEChMSQ2VydGlmaWNhdG" +
                    "UgSXNzdWVyMB4XDTcwMDExMjEzNDY0MFoXDTcwMDEyNDAzMzMy" +
                    "MFowHzEdMBsGA1UEChMUU3ViamVjdCBPcmdhbml6YXRpb24wgg" +
                    "G4MIIBLAYHKoZIzjgEATCCAR8CgYEA/X9TgR11EilS30qcLuzk" +
                    "5/YRt1I870QAwx4/gLZRJmlFXUAiUftZPY1Y+r/F9bow9subVW" +
                    "zXgTuAHTRv8mZgt2uZUKWkn5/oBHsQIsJPu6nX/rfGG/g7V+fG" +
                    "qKYVDwT7g/bTxR7DAjVUE1oWkTL2dfOuK2HXKu/yIgMZndFIAc" +
                    "cCFQCXYFCPFSMLzLKSuYKi64QL8Fgc9QKBgQD34aCF1ps93su8" +
                    "q1w2uFe5eZSvu/o66oL5V0wLPQeCZ1FZV4661FlP5nEHEIGAtE" +
                    "kWcSPoTCgWE7fPCTKMyKbhPBZ6i1R8jSjgo64eK7OmdZFuo38L" +
                    "+iE1YvH7YnoBJDvMpPG+qFGQiaiD3+Fa5Z8GkotmXoB7VSVkAU" +
                    "w7/s9JKgOBhQACgYEAzhsiNfq3U9RAWhut8YDmRbHJTXMHTytG" +
                    "sK0iRZqg47JllyWbHwSmS8T2k4Dk4FYxQzxgOxW/PbZ5c06P9B" +
                    "YPlazQTcPIwAWSlKTaoTfnWzHnp75hbbPc8HmrSIEHVPr5hHS0" +
                    "DQl5Fu45b5U2KG5cx2Bb3lk17n0y/RqFXFYTv+OBAgCqggIAVa" +
                    "OCAhQwggIQMA8GA1UdDwEB/wQFAwMBqoAwEgYDVR0TAQH/BAgw" +
                    "BgEB/wIBBTAUBgNVHSABAf8ECjAIMAYGBFUdIAAwZwYDVR0RAQ" +
                    "H/BF0wW4EMcmZjQDgyMi5OYW1lggdkTlNOYW1lpBcxFTATBgNV" +
                    "BAoTDE9yZ2FuaXphdGlvboYaaHR0cDovL3VuaWZvcm0uUmVzb3" +
                    "VyY2UuSWSHBP///wCIByoDolyDsgMwDAYDVR0eAQH/BAIwADAM" +
                    "BgNVHSQBAf8EAjAAMIGZBgNVHSUBAf8EgY4wgYsGBFUdJQAGCC" +
                    "sGAQUFBwMBBggrBgEFBQcDAQYIKwYBBQUHAwIGCCsGAQUFBwMD" +
                    "BggrBgEFBQcDBAYIKwYBBQUHAwUGCCsGAQUFBwMGBggrBgEFBQ" +
                    "cDBwYIKwYBBQUHAwgGCCsGAQUFBwMJBggrBgEFBQgCAgYKKwYB" +
                    "BAGCNwoDAwYJYIZIAYb4QgQBMA0GA1UdNgEB/wQDAgEBMA4GBC" +
                    "pNhgkBAf8EAwEBATBkBgNVHRIEXTBbgQxyZmNAODIyLk5hbWWC" +
                    "B2ROU05hbWWkFzEVMBMGA1UEChMMT3JnYW5pemF0aW9uhhpodH" +
                    "RwOi8vdW5pZm9ybS5SZXNvdXJjZS5JZIcE////AIgHKgOiXIOy" +
                    "AzAJBgNVHR8EAjAAMAoGA1UdIwQDAQEBMAoGA1UdDgQDAQEBMA" +
                    "oGA1UdIQQDAQEBMAwGByqGSM44BAMBAQADMAAwLQIVAIu6VjdQ" +
                    "kOyX3jwUCvZk7i8ascXxAhQb4oUMkiRQ1/ThtGvgtiuQUzHBZA" +
                    "==";

    // 2 consecutively encoded X.509 certificates
    private static String certEncodingBase64 =
            "MIIC+jCCAragAwIBAgICAiswDAYHKoZIzjgEAwEBADAdMRswGQ" +
                    "YDVQQKExJDZXJ0aWZpY2F0ZSBJc3N1ZXIwIhgPMTk3MDAxMTIx" +
                    "MzQ2NDBaGA8xOTcwMDEyNDAzMzMyMFowHzEdMBsGA1UEChMUU3" +
                    "ViamVjdCBPcmdhbml6YXRpb24wGTAMBgcqhkjOOAQDAQEAAwkA" +
                    "AQIDBAUGBwiBAgCqggIAVaOCAhQwggIQMA8GA1UdDwEB/wQFAw" +
                    "MBqoAwEgYDVR0TAQH/BAgwBgEB/wIBBTAUBgNVHSABAf8ECjAI" +
                    "MAYGBFUdIAAwZwYDVR0RAQH/BF0wW4EMcmZjQDgyMi5OYW1lgg" +
                    "dkTlNOYW1lpBcxFTATBgNVBAoTDE9yZ2FuaXphdGlvboYaaHR0" +
                    "cDovL3VuaWZvcm0uUmVzb3VyY2UuSWSHBP///wCIByoDolyDsg" +
                    "MwDAYDVR0eAQH/BAIwADAMBgNVHSQBAf8EAjAAMIGZBgNVHSUB" +
                    "Af8EgY4wgYsGBFUdJQAGCCsGAQUFBwMBBggrBgEFBQcDAQYIKw" +
                    "YBBQUHAwIGCCsGAQUFBwMDBggrBgEFBQcDBAYIKwYBBQUHAwUG" +
                    "CCsGAQUFBwMGBggrBgEFBQcDBwYIKwYBBQUHAwgGCCsGAQUFBw" +
                    "MJBggrBgEFBQgCAgYKKwYBBAGCNwoDAwYJYIZIAYb4QgQBMA0G" +
                    "A1UdNgEB/wQDAgEBMA4GBCpNhgkBAf8EAwEBATBkBgNVHRIEXT" +
                    "BbgQxyZmNAODIyLk5hbWWCB2ROU05hbWWkFzEVMBMGA1UEChMM" +
                    "T3JnYW5pemF0aW9uhhpodHRwOi8vdW5pZm9ybS5SZXNvdXJjZS" +
                    "5JZIcE////AIgHKgOiXIOyAzAJBgNVHR8EAjAAMAoGA1UdIwQD" +
                    "AQEBMAoGA1UdDgQDAQEBMAoGA1UdIQQDAQEBMAwGByqGSM44BA" +
                    "MBAQADMAAwLQIUAL4QvoazNWP7jrj84/GZlhm09DsCFQCBKGKC" +
                    "GbrP64VtUt4JPmLjW1VxQDCCAvowggK2oAMCAQICAgIrMAwGBy" +
                    "qGSM44BAMBAQAwHTEbMBkGA1UEChMSQ2VydGlmaWNhdGUgSXNz" +
                    "dWVyMCIYDzE5NzAwMTEyMTM0NjQwWhgPMTk3MDAxMjQwMzMzMj" +
                    "BaMB8xHTAbBgNVBAoTFFN1YmplY3QgT3JnYW5pemF0aW9uMBkw" +
                    "DAYHKoZIzjgEAwEBAAMJAAECAwQFBgcIgQIAqoICAFWjggIUMI" +
                    "ICEDAPBgNVHQ8BAf8EBQMDAaqAMBIGA1UdEwEB/wQIMAYBAf8C" +
                    "AQUwFAYDVR0gAQH/BAowCDAGBgRVHSAAMGcGA1UdEQEB/wRdMF" +
                    "uBDHJmY0A4MjIuTmFtZYIHZE5TTmFtZaQXMRUwEwYDVQQKEwxP" +
                    "cmdhbml6YXRpb26GGmh0dHA6Ly91bmlmb3JtLlJlc291cmNlLk" +
                    "lkhwT///8AiAcqA6Jcg7IDMAwGA1UdHgEB/wQCMAAwDAYDVR0k" +
                    "AQH/BAIwADCBmQYDVR0lAQH/BIGOMIGLBgRVHSUABggrBgEFBQ" +
                    "cDAQYIKwYBBQUHAwEGCCsGAQUFBwMCBggrBgEFBQcDAwYIKwYB" +
                    "BQUHAwQGCCsGAQUFBwMFBggrBgEFBQcDBgYIKwYBBQUHAwcGCC" +
                    "sGAQUFBwMIBggrBgEFBQcDCQYIKwYBBQUIAgIGCisGAQQBgjcK" +
                    "AwMGCWCGSAGG+EIEATANBgNVHTYBAf8EAwIBATAOBgQqTYYJAQ" +
                    "H/BAMBAQEwZAYDVR0SBF0wW4EMcmZjQDgyMi5OYW1lggdkTlNO" +
                    "YW1lpBcxFTATBgNVBAoTDE9yZ2FuaXphdGlvboYaaHR0cDovL3" +
                    "VuaWZvcm0uUmVzb3VyY2UuSWSHBP///wCIByoDolyDsgMwCQYD" +
                    "VR0fBAIwADAKBgNVHSMEAwEBATAKBgNVHQ4EAwEBATAKBgNVHS" +
                    "EEAwEBATAMBgcqhkjOOAQDAQEAAzAAMC0CFAC+EL6GszVj+464" +
                    "/OPxmZYZtPQ7AhUAgShighm6z+uFbVLeCT5i41tVcUA=";

    private static String base64certEncoding =
            "-----BEGIN CERTIFICATE-----\n" +
                    "MIIC+jCCAragAwIBAgICAiswDAYHKoZIzjgEAwEBADAdMRswGQYDVQQKExJDZXJ0a" +
                    "WZpY2F0ZSBJc3N1ZXIwIhgPMTk3MDAxMTIxMzQ2NDBaGA8xOTcwMDEyNDAzMzMyMF" +
                    "owHzEdMBsGA1UEChMUU3ViamVjdCBPcmdhbml6YXRpb24wGTAMBgcqhkjOOAQDAQE" +
                    "AAwkAAQIDBAUGBwiBAgCqggIAVaOCAhQwggIQMA8GA1UdDwEB/wQFAwMBqoAwEgYD" +
                    "VR0TAQH/BAgwBgEB/wIBBTAUBgNVHSABAf8ECjAIMAYGBFUdIAAwZwYDVR0RAQH/B" +
                    "F0wW4EMcmZjQDgyMi5OYW1lggdkTlNOYW1lpBcxFTATBgNVBAoTDE9yZ2FuaXphdG" +
                    "lvboYaaHR0cDovL3VuaWZvcm0uUmVzb3VyY2UuSWSHBP///wCIByoDolyDsgMwDAY" +
                    "DVR0eAQH/BAIwADAMBgNVHSQBAf8EAjAAMIGZBgNVHSUBAf8EgY4wgYsGBFUdJQAG" +
                    "CCsGAQUFBwMBBggrBgEFBQcDAQYIKwYBBQUHAwIGCCsGAQUFBwMDBggrBgEFBQcDB" +
                    "AYIKwYBBQUHAwUGCCsGAQUFBwMGBggrBgEFBQcDBwYIKwYBBQUHAwgGCCsGAQUFBw" +
                    "MJBggrBgEFBQgCAgYKKwYBBAGCNwoDAwYJYIZIAYb4QgQBMA0GA1UdNgEB/wQDAgE" +
                    "BMA4GBCpNhgkBAf8EAwEBATBkBgNVHRIEXTBbgQxyZmNAODIyLk5hbWWCB2ROU05h" +
                    "bWWkFzEVMBMGA1UEChMMT3JnYW5pemF0aW9uhhpodHRwOi8vdW5pZm9ybS5SZXNvd" +
                    "XJjZS5JZIcE////AIgHKgOiXIOyAzAJBgNVHR8EAjAAMAoGA1UdIwQDAQEBMAoGA1" +
                    "UdDgQDAQEBMAoGA1UdIQQDAQEBMAwGByqGSM44BAMBAQADMAAwLQIUAL4QvoazNWP" +
                    "7jrj84/GZlhm09DsCFQCBKGKCGbrP64VtUt4JPmLjW1VxQA==\n" +
                    "-----END CERTIFICATE-----\n" +
                    "-----BEGIN CERTIFICATE-----\n" +
                    "MIIC+jCCAragAwIBAgICAiswDAYHKoZIzjgEAwEBADAdMRswGQYDVQQKExJDZXJ0a" +
                    "WZpY2F0ZSBJc3N1ZXIwIhgPMTk3MDAxMTIxMzQ2NDBaGA8xOTcwMDEyNDAzMzMyMF" +
                    "owHzEdMBsGA1UEChMUU3ViamVjdCBPcmdhbml6YXRpb24wGTAMBgcqhkjOOAQDAQE" +
                    "AAwkAAQIDBAUGBwiBAgCqggIAVaOCAhQwggIQMA8GA1UdDwEB/wQFAwMBqoAwEgYD" +
                    "VR0TAQH/BAgwBgEB/wIBBTAUBgNVHSABAf8ECjAIMAYGBFUdIAAwZwYDVR0RAQH/B" +
                    "F0wW4EMcmZjQDgyMi5OYW1lggdkTlNOYW1lpBcxFTATBgNVBAoTDE9yZ2FuaXphdG" +
                    "lvboYaaHR0cDovL3VuaWZvcm0uUmVzb3VyY2UuSWSHBP///wCIByoDolyDsgMwDAY" +
                    "DVR0eAQH/BAIwADAMBgNVHSQBAf8EAjAAMIGZBgNVHSUBAf8EgY4wgYsGBFUdJQAG" +
                    "CCsGAQUFBwMBBggrBgEFBQcDAQYIKwYBBQUHAwIGCCsGAQUFBwMDBggrBgEFBQcDB" +
                    "AYIKwYBBQUHAwUGCCsGAQUFBwMGBggrBgEFBQcDBwYIKwYBBQUHAwgGCCsGAQUFBw" +
                    "MJBggrBgEFBQgCAgYKKwYBBAGCNwoDAwYJYIZIAYb4QgQBMA0GA1UdNgEB/wQDAgE" +
                    "BMA4GBCpNhgkBAf8EAwEBATBkBgNVHRIEXTBbgQxyZmNAODIyLk5hbWWCB2ROU05h" +
                    "bWWkFzEVMBMGA1UEChMMT3JnYW5pemF0aW9uhhpodHRwOi8vdW5pZm9ybS5SZXNvd" +
                    "XJjZS5JZIcE////AIgHKgOiXIOyAzAJBgNVHR8EAjAAMAoGA1UdIwQDAQEBMAoGA1" +
                    "UdDgQDAQEBMAoGA1UdIQQDAQEBMAwGByqGSM44BAMBAQADMAAwLQIUAL4QvoazNWP" +
                    "7jrj84/GZlhm09DsCFQCBKGKCGbrP64VtUt4JPmLjW1VxQA==\n" +
                    "-----END CERTIFICATE-----\n";

    private static String crlEncodingBase64 =
            "MIIBNDCB8gIBATAMBgcqhkjOOAQDAQEAMBUxEzARBgNVBAoTCk" +
                    "NSTCBJc3N1ZXIXDTA1MDcxMzA5NDkzMFoXDTA1MDcxMzA5NTEx" +
                    "MFowgZgwEwICAisXDTA1MDcxMzA5NDkzMVowbAICAjcXDTA1MD" +
                    "cxMzA5NDkzMVowVzAKBgNVHRUEAwoBATAcBgNVHRgEFRgTMjAw" +
                    "NTA3MTMwOTQ5MzAuNTYyWjArBgNVHR0BAf8EITAfpB0xGzAZBg" +
                    "NVBAoTEkNlcnRpZmljYXRlIElzc3VlcjATAgIDCRcNMDUwNzEz" +
                    "MDk0OTMxWqAPMA0wCwYDVR0UBAQCAhFcMAwGByqGSM44BAMBAQ" +
                    "ADLwAwLAIUYp/1MGXvQ1a/wF+WvmRGnQHJ+nQCFBJ4JFNfcCN7" +
                    "wNUpSOnUYHEC8uInMIIBNDCB8gIBATAMBgcqhkjOOAQDAQEAMB" +
                    "UxEzARBgNVBAoTCkNSTCBJc3N1ZXIXDTA1MDcxMzA5NDkzMFoX" +
                    "DTA1MDcxMzA5NTExMFowgZgwEwICAisXDTA1MDcxMzA5NDkzMV" +
                    "owbAICAjcXDTA1MDcxMzA5NDkzMVowVzAKBgNVHRUEAwoBATAc" +
                    "BgNVHRgEFRgTMjAwNTA3MTMwOTQ5MzAuNTYyWjArBgNVHR0BAf" +
                    "8EITAfpB0xGzAZBgNVBAoTEkNlcnRpZmljYXRlIElzc3VlcjAT" +
                    "AgIDCRcNMDUwNzEzMDk0OTMxWqAPMA0wCwYDVR0UBAQCAhFcMA" +
                    "wGByqGSM44BAMBAQADLwAwLAIUYp/1MGXvQ1a/wF+WvmRGnQHJ" +
                    "+nQCFBJ4JFNfcCN7wNUpSOnUYHEC8uIn";

    private static String crlEncodingPEM =
            "-----BEGIN X509 CRL-----\n" +
                    "MIIBNDCB8gIBATAMBgcqhkjOOAQDAQEAMBUxEzARBgNVBAoTCk" +
                    "NSTCBJc3N1ZXIXDTA1MDcxMzA5NDkzMFoXDTA1MDcxMzA5NTEx" +
                    "MFowgZgwEwICAisXDTA1MDcxMzA5NDkzMVowbAICAjcXDTA1MD" +
                    "cxMzA5NDkzMVowVzAKBgNVHRUEAwoBATAcBgNVHRgEFRgTMjAw" +
                    "NTA3MTMwOTQ5MzAuNTYyWjArBgNVHR0BAf8EITAfpB0xGzAZBg" +
                    "NVBAoTEkNlcnRpZmljYXRlIElzc3VlcjATAgIDCRcNMDUwNzEz" +
                    "MDk0OTMxWqAPMA0wCwYDVR0UBAQCAhFcMAwGByqGSM44BAMBAQ" +
                    "ADLwAwLAIUYp/1MGXvQ1a/wF+WvmRGnQHJ+nQCFBJ4JFNfcCN7" +
                    "wNUpSOnUYHEC8uIm\n" +
                    "-----END X509 CRL-----\n" +
                    "-----BEGIN X509 CRL-----\n" +
                    "MIIBNDCB8gIBATAMBgcqhkjOOAQDAQEAMB" +
                    "UxEzARBgNVBAoTCkNSTCBJc3N1ZXIXDTA1MDcxMzA5NDkzMFoX" +
                    "DTA1MDcxMzA5NTExMFowgZgwEwICAisXDTA1MDcxMzA5NDkzMV" +
                    "owbAICAjcXDTA1MDcxMzA5NDkzMVowVzAKBgNVHRUEAwoBATAc" +
                    "BgNVHRgEFRgTMjAwNTA3MTMwOTQ5MzAuNTYyWjArBgNVHR0BAf" +
                    "8EITAfpB0xGzAZBgNVBAoTEkNlcnRpZmljYXRlIElzc3VlcjAT" +
                    "AgIDCRcNMDUwNzEzMDk0OTMxWqAPMA0wCwYDVR0UBAQCAhFcMA" +
                    "wGByqGSM44BAMBAQADLwAwLAIUYp/1MGXvQ1a/wF+WvmRGnQHJ" +
                    "+nQCFBJ4JFNfcCN7wNUpSOnUYHEC8uIo\n" +
                    "-----END X509 CRL-----\n";


    public static byte[] getCertPathPKCS7Encoding() throws UnsupportedEncodingException {
        return Base64.decode(certPathPKCS7Base64.getBytes("UTF-8"));
    }

    public static byte[] getCertPathPkiPathEncoding() throws UnsupportedEncodingException {
        return Base64.decode(certPathPkiPathBase64.getBytes("UTF-8"));
    }

    public static byte[] getCertEncoding() throws UnsupportedEncodingException {
        return Base64.decode(certEncodingBase64.getBytes("UTF-8"));
    }

    public static byte[] getBase64CertEncoding() throws UnsupportedEncodingException {
        return base64certEncoding.getBytes("UTF-8");
    }

    public static byte[] getBase64CRLEncoding() throws UnsupportedEncodingException {
        return crlEncodingPEM.getBytes("UTF-8");
    }

    public static byte[] getCRLEncoding() throws UnsupportedEncodingException {
        return Base64.decode(crlEncodingBase64.getBytes("UTF-8"));
    }
}

